#!/usr/bin/perl -w

use CGI 'header', 'param';
use CGI::Carp 'fatalsToBrowser';
$CGI::POST_MAX=1024 * 500;  # max 500K posts
$CGI::DISABLE_UPLOADS = 1;  # no uploads     
$query = new CGI;   # Setup CGI object

##
## quickmblog (Quick Multiuser Blogger)
## Written by Aaron Hachez, 2003
## Released under GPL (Gnu General Public License)    
## See included file LICENSE.txt for more details, or
## Visit http://www.gnu.org for a copy.
##
## add.pl
## 
## Purpose: To allow a user after authentication to create an entry for the "blog". Files
## have a naming convention of <year>.<month>.<day>.<hour>.<minute>.<second>.quickmblog
##

####################
### Beg Varibles ###

# The allowed_hosts and denied_hosts use the following syntax
#
# You must use ip addresses, excepting a single "*" which indicates
# match all.  You can use a partial IP address (i.e. 10.0., or 10.0.0.),
# or a "*" to match all. Allow is checked first, if the IP is in the
# allow list, or allow is set to it's default "*", no further checking is done.
# if an IP is not in the allow list, the deny list is checked next.
# If the IP is in neither list, and neither list is using the "*" wildcard,
# the user is let through.
#
# In this example, only users coming from addresses begining
# with "10.0." or "192.168.0." are allowed through, and all others are denied.
#
# @allowed_hosts = ("10.0.", "192.168.0.");
# @denied_hosts = ("*");
#
# In this example, allow is set to match all, so everyone is allowed, and the deny
# list is not checked.
#
# @allowed_hosts = ("*");
# @denied_hosts = ("10.0.0.", "192.168.");
#
# In this example, allow is set to nothing, so only those addresses listed in
# the deny list are refused. Setting the deny list to nothing is effectively
# the same as setting the allow to "*".
#
# @allowed_hosts = ("");
# @denied_hosts = ("10.0.0.", "192.168.");

# An allowed hosts list
@allowed_hosts = ("*");

# An denied hosts list
@denied_hosts = ("*");

#########################################
## You probably don't need to change   ##
## Anything below this point, but can. ##
#########################################

# The title for this script's output
$title = "Add entry:";

# The name/url path of this script
$self = "add.pl";

# The show script
$show_script = "show.pl";

# The password file to use
$password_file = ".pass";

# The path to user entries
$entries_path = "./entries/";

# The path to display templates
$template_path = "./templates/";

# The template filename to use
$template_file = "manage.html";

### End Varibles ###
####################
################
### BEG MAIN ###

# Obligitory content type statment
print "Content-type: text/html\n\n";

# Initially set allowed to false
my($allowed) = 0;

# Before we do anything else, check the remote host(s)
# Check first to see if allowed, if not in the allowed list, Check the denied list.
# If they are in the allow list
if(check_allowed_hosts($query->remote_addr()))
{
	$allowed = 1;
}
# If they are _not_ in the denied list
elsif(!check_denied_hosts($query->remote_addr()))
{
	$allowed = 1;
}

# If the caller was either in the allow list, or not in the deny list
if($allowed eq 1)
{
	my($result);
	my($create_result) = "";
	my($user) = param('user');
	my($pass) = param('pass');
	my($text) = param('text');

	# Get username, password and the desired text, if success, attempt to create entry
	if(defined($user) and defined($pass) and defined($text))
	{
		# Check supplied username and password
		# We store the result so that we can supply meaning full
		# messages in the template output below.
		if(($result = check_pass($user, $pass)) eq 2)
		{
			# If valid, create the new entry
			$create_result = create_entry($user, $text);
		}
	}

	# Get the template for managment, and output useful errors
	@template = get_template($template_file);

	# Go through each line, and replace user or text tags with the data
	# passed to this script, so the user doesn't have to re-enter it
	# Also replace mesages tag with relevant error.
	foreach $arg (@template)
	{
		my($message);

		# If result is undefined
		if(defined($result))
		{
			# One means bad password
			if($result eq 1)
			{
				$message = "Please ensure that you have entered a valid username, password, and a message.";
			}
			# Two means success
			elsif($result eq 2)
			{
				if($create_result ne "")
				{
					$message = "Your message has been posted successfully.";
				}
				else
				{
					$message = "Message post failed, please ensure that you have entered a valid username, password, and a message.";
				}
			}
			elsif($result eq 0)
			{
				$message = "Please ensure that you have entered a valid username, password, and a message.";
			}
			else
			{
				$message = "Got strange result: $result";
			}
		}
		else
		{
			# If result is not defined, there wasn't enough info.

			# If any of these is defined, someone tried to post and failed.
			# This is semi redundant, but better safe than sorry.
			if(defined($user) or defined($pass) or defined($text))
			{
				$message = "Please ensure that you have entered a username, password and a message.";
			}
			# Else display a default entry
			else
			{
				$message = "Add a new entry.";
			}
		}

		$arg =~ s/^(.*)\<\!--\$TITLE\$--\>(.*)$/$1$title$2/g;

		$arg =~ s/^(.*)\<\!--\$MESSAGE\$--\>(.*)$/$1$message$2/g;

		$arg =~ s/^(.*)\<\!--\$ACTION\$--\>(.*)$/$1$self$2/g;

		# If a username was entered, replace tag, else remove tag
		if(defined($user))
		{
			$arg =~ s/^(.*)\<\!--\$USER\$--\>(.*$)/$1$user$2/g;
		}
		else
		{
			$arg =~ s/^(.*)\<\!--\$USER\$--\>(.*$)/$1$2/g;
		}

		# If text was entered, replace tag, else remove tag
		if(defined($text))
		{
			$arg =~ s/^(.*)\<\!--\$TEXT\$--\>(.*)$/$1$text$2/g;
		}
		else
		{
			$arg =~ s/^(.*)\<\!--\$TEXT\$--\>(.*)$/$1$2/g;
		}

		$arg =~ s/^(.*)\<\!--\$SHOW\$--\>(.*)$/$1$show_script$2/g;

		print $arg;
	}
}
# Else caller is not allowed to access
else
{
	print "You have been denied access.\n";
}

### END MAIN ###
################
#####################
### Sub Functions ###

## Get a template file for editing entries
# takes one arg, filename
sub get_template
{
	if(!$_[0])
	{
		die "No filename passed for template.\n";
	}

	# Get the file name and path from first arg passed
	my($filename) = $_[0];
	
	# Open template file for read
	open(TEMPLATE, "$template_path/$filename") || die "Couldn't find template file: $filename\n";

	# Get entire template file contents
	my(@template) = <TEMPLATE>;

	# Close template file
	close(TEMPLATE);

	# Return template file contents
	return(@template);
}

## Check the suplied username/password to make sure it's valid
# returns 1 if matched user, but password was wrong, returns 2 if matched user and password, returns 0 if neither.
sub check_pass
{
	# Set result to no match
	my($result) = 0;

	my($user_from_file);
	my($pass_from_file);

	if((my($user) = $_[0]) and (my($pass) = $_[1]))
	{
		if($user ne "" and $pass ne "")
		{

		open(FILE, "$password_file") || die "Couldn't open file: $password_file\n";

		## For each line in the file
		foreach $arg (<FILE>)
		{
			# Remove trailing \n
			chomp($arg);

			# Split username/password pair out at :
			($user_from_file, $pass_from_file) = split(/:/, $arg);
			
			# If the username is a match for the username passed
			if($user eq $user_from_file)
			{
				# Set result to matched user
				$result = 1;

				# Exit foreach
				last;
			}
		}
		
		close(FILE);

		# Debug
		#print("$user_from_file $user $pass_from_file $pass\n");

		# Chomp off extra \n's
		chomp($pass);

		# Compare password from input to password on file
		if(crypt($pass, $pass_from_file) eq $pass_from_file)
		{
			# Set result to matched password
			$result = 2;
		}

		}
	}
	else
	{
		print "Error, no username and/or password supplied.\n";
	}

	return($result);
}

# Create a new entry
sub create_entry
{
	my($user);
	my($text);
	my($filename) = "";

	# Make sure that we were passed valid data
	if(($user = $_[0]) and ($text = $_[1]))
	{
		# Get a current time
		my($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);

		# Format year as full year, default is number of years since 1900
		$year = (1900 + $year);

		# Format month as two digits
		$mon = sprintf("%0.2d", ($mon + 1));

		# Format day as two digits
		$mday = sprintf("%0.2d", $mday);

		# Format minutes as two digits
		$hour = sprintf("%0.2d", $hour);
		
		# Format minutes as two digits
		$min = sprintf("%0.2d", $min);
		
		# Format minutes as two digits
		$sec = sprintf("%0.2d", $sec);

		$filename = $year . "." . $mon . "." . $mday . "." . $hour . "." . $min . "." . $sec . ".quickmblog." . $user;
		
		# Debug
		#print "$filename\n";

		# Open file for write
		open(FILE, ">>$entries_path/$filename") || die "Unable to open file for append: $entries_path/$filename\n";

		# The following line is for control data, date appended every time the file has been modified.
		print FILE "$year.$mon.$mday\n";

		print FILE "Posted by $user on $year.$mon.$mday at $hour:$min:$sec.\n\n";

		print FILE "$text";

		close(FILE);
	}

	return($filename);
}

# Check the @allowed_hosts list to see if current caller is allowed
sub check_allowed_hosts
{
	my($result) = 0;

	# If an argument has been passed
	if(defined($_[0]))
	{
		# Get arg in local variable
		my($remote_host) = $_[0];

		# Debug
		#print "Allowedlist: @allowed_hosts, remotehost: $remote_host\n";

		# If our allowed hosts list first element eq a single "*" indicating all
		if($allowed_hosts[0] eq "*")
		{
			$result = 1;
		
			# Debug
			#print "ALLOWED BY *";
		}
		else
		{
			# Loop through the allowed list
			foreach $arg (@allowed_hosts)
			{
				# If arg is not blank
				if($arg ne "")
				{
					# If the remote matches 
					if($remote_host =~ /^$arg.*$/)
					{
						# Debug
						#print "ALLOWED BY MATCH";

						$result = 1;
						last;
					}
				}
			}
		}
	}

	return($result);
}

# Check the @denied_hosts list to see if current caller is denied
sub check_denied_hosts
{
	my($result) = 0;

	# If an argument has been passed
	if(defined($_[0]))
	{
		# Get arg in local variable
		my($remote_host) = $_[0];

		# Debug
		#print "Deniedlist: @denied_hosts, remotehost: $remote_host\n";

		# If our denied hosts list first element eq a single "*" indicating all
		if($denied_hosts[0] eq "*")
		{
			$result = 1;
			
			# Debug
			#print "DENIED BY *";
		}
		else
		{
			# Loop through the allowed list
			foreach $arg (@denied_hosts)
			{
				# If arg is not blank
				if($arg ne "")
				{
					# If the remote matches 
					if($remote_host =~ /^$arg.*$/)
					{
						# Debug
						#print "DENIED BY MATCH";

						$result = 1;
						last;
					}
				}
			}
		}
	}

	return($result);
}
